@@ -1,12 +1,18 @@
 /*
- * Copyright (c) 2002-2007, Communications and Remote Sensing Laboratory, Universite catholique de Louvain (UCL), Belgium
- * Copyright (c) 2002-2007, Professor Benoit Macq
+ * The copyright in this software is being made available under the 2-clauses 
+ * BSD License, included below. This software may be subject to other third 
+ * party and contributor rights, including patent rights, and no such rights
+ * are granted under this license.
+ *
+ * Copyright (c) 2002-2014, Universite catholique de Louvain (UCL), Belgium
+ * Copyright (c) 2002-2014, Professor Benoit Macq
  * Copyright (c) 2001-2003, David Janssens
  * Copyright (c) 2002-2003, Yannick Verschueren
- * Copyright (c) 2003-2007, Francois-Olivier Devaux and Antonin Descampe
+ * Copyright (c) 2003-2007, Francois-Olivier Devaux 
+ * Copyright (c) 2003-2014, Antonin Descampe
  * Copyright (c) 2005, Herve Drolon, FreeImage Team
  * Copyright (c) 2006-2007, Parvatha Elangovan
- * Copyright (c) 2008;2011-2012, Centre National d'Etudes Spatiales (CNES), France 
+ * Copyright (c) 2008, 2011-2012, Centre National d'Etudes Spatiales (CNES), FR 
  * Copyright (c) 2012, CS Systemes d'Information, France
  * All rights reserved.
  *
@@ -31,7 +37,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
-#include "opj_config.h"
+#include "opj_apps_config.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -57,10 +63,10 @@
 #include "convert.h"
 #include "index.h"
 
-#ifdef HAVE_LIBLCMS2
+#ifdef OPJ_HAVE_LIBLCMS2
 #include <lcms2.h>
 #endif
-#ifdef HAVE_LIBLCMS1
+#ifdef OPJ_HAVE_LIBLCMS1
 #include <lcms.h>
 #endif
 #include "color.h"
@@ -100,31 +106,25 @@ int parse_DA_values( char* inArg, unsigned int *DA_x0, unsigned int *DA_y0, unsi
 
 /* -------------------------------------------------------------------------- */
 static void decode_help_display(void) {
-	fprintf(stdout,"HELP for opj_decompress\n----\n\n");
-	fprintf(stdout,"- the -h option displays this help information on screen\n\n");
+    fprintf(stdout,"\nThis is the opj_decompress utility from the OpenJPEG project.\n"
+            "It decompresses JPEG 2000 codestreams to various image formats.\n"
+            "It has been compiled against openjp2 library v%s.\n\n",opj_version());
 
-/* UniPG>> */
-	fprintf(stdout,"List of parameters for the JPEG 2000 "
-#ifdef USE_JPWL
-		"+ JPWL "
-#endif /* USE_JPWL */
-		"decoder:\n");
-/* <<UniPG */
+    fprintf(stdout,"Parameters:\n");
+    fprintf(stdout,"-----------\n");
 	fprintf(stdout,"\n");
-	fprintf(stdout,"\n");
-	fprintf(stdout,"  -ImgDir \n");
+    fprintf(stdout,"  -ImgDir <directory> \n");
 	fprintf(stdout,"	Image file Directory path \n");
-	fprintf(stdout,"  -OutFor \n");
+    fprintf(stdout,"  -OutFor <PBM|PGM|PPM|PNM|PAM|PGX|PNG|BMP|TIF|RAW|RAWL|TGA>\n");
 	fprintf(stdout,"    REQUIRED only if -ImgDir is used\n");
-	fprintf(stdout,"	  Need to specify only format without filename <BMP>  \n");
-	fprintf(stdout,"    Currently accepts PGM, PPM, PNM, PGX, PNG, BMP, TIF, RAW and TGA formats\n");
+    fprintf(stdout,"	Output format for decompressed images.\n");
 	fprintf(stdout,"  -i <compressed file>\n");
-	fprintf(stdout,"    REQUIRED only if an Input image directory not specified\n");
+    fprintf(stdout,"    REQUIRED only if an Input image directory is not specified\n");
 	fprintf(stdout,"    Currently accepts J2K-files, JP2-files and JPT-files. The file type\n");
 	fprintf(stdout,"    is identified based on its suffix.\n");
 	fprintf(stdout,"  -o <decompressed file>\n");
 	fprintf(stdout,"    REQUIRED\n");
-	fprintf(stdout,"    Currently accepts PGM, PPM, PNM, PGX, PNG, BMP, TIF, RAW and TGA files\n");
+    fprintf(stdout,"    Currently accepts formats specified above (see OutFor option)\n");
 	fprintf(stdout,"    Binary data is written to the file (not ascii). If a PGX\n");
 	fprintf(stdout,"    filename is given, there will be as many output files as there are\n");
 	fprintf(stdout,"    components: an indice starting from 0 will then be appended to the\n");
@@ -222,7 +222,7 @@ int get_file_format(const char *filename) {
 	if (ext == NULL)
 		return -1;
 	ext++;
-	if(ext) {
+	if(*ext) {
 		for(i = 0; i < sizeof(format)/sizeof(*format); i++) {
 			if(strcasecmp(ext, extension[i]) == 0) {
 				return format[i];
@@ -271,7 +271,7 @@ static int infile_format(const char *fname)
 	const char *s, *magic_s;
 	int ext_format, magic_format;
 	unsigned char buf[12];
-	unsigned int l_nb_read;
+	OPJ_SIZE_T l_nb_read;
 
 	reader = fopen(fname, "rb");
 
@@ -325,7 +325,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 	int totlen, c;
 	opj_option_t long_option[]={
 		{"ImgDir",REQ_ARG, NULL ,'y'},
-		{"OutFor",REQ_ARG, NULL ,'O'},
+        {"OutFor",REQ_ARG, NULL ,'O'}
 	};
 
 	const char optlist[] = "i:o:r:l:x:d:t:"
@@ -335,7 +335,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 					"W:"
 #endif /* USE_JPWL */
 /* <<UniPG */
-			"h"		;
+            "h"		;
 	totlen=sizeof(long_option);
 	img_fol->set_out_format = 0;
 	do {
@@ -361,7 +361,8 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 						return 1;
 					default:
 						fprintf(stderr, 
-							"!! Unrecognized format for infile: %s [accept only *.j2k, *.jp2, *.jpc or *.jpt] !!\n\n", 
+                            "[ERROR] Unknown input file format: %s \n"
+                            "        Known file formats are *.j2k, *.jp2, *.jpc or *.jpt\n",
 							infile);
 						return 1;
 				}
@@ -447,7 +448,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 
 			case 'r':		/* reduce option */
 			{
-				sscanf(opj_optarg, "%d", &parameters->cp_reduce);
+				sscanf(opj_optarg, "%ud", &parameters->cp_reduce);
 			}
 			break;
 			
@@ -456,7 +457,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 
 			case 'l':		/* layering option */
 			{
-				sscanf(opj_optarg, "%d", &parameters->cp_layer);
+				sscanf(opj_optarg, "%ud", &parameters->cp_layer);
 			}
 			break;
 			
@@ -466,10 +467,10 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 				decode_help_display();
 				return 1;				
 
-				/* ------------------------------------------------------ */
+            /* ----------------------------------------------------- */
 
 			case 'y':			/* Image Directory path */
-				{
+                {
 					img_fol->imgdirpath = (char*)malloc(strlen(opj_optarg) + 1);
 					strcpy(img_fol->imgdirpath,opj_optarg);
 					img_fol->set_imgdir=1;
@@ -481,7 +482,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 			case 'd':     		/* Input decode ROI */
 			{
 				int size_optarg = (int)strlen(opj_optarg) + 1;
-				char *ROI_values = (char*) malloc(size_optarg);
+				char *ROI_values = (char*) malloc((size_t)size_optarg);
 				ROI_values[0] = '\0';
 				strncpy(ROI_values, opj_optarg, strlen(opj_optarg));
 				ROI_values[strlen(opj_optarg)] = '\0';
@@ -496,7 +497,7 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 
 			case 't':     		/* Input tile index */
 			{
-				sscanf(opj_optarg, "%d", &parameters->tile_index);
+				sscanf(opj_optarg, "%ud", &parameters->tile_index);
 				parameters->nb_tile_to_decode = 1;
 			}
 			break;
@@ -583,31 +584,33 @@ int parse_cmdline_decoder(int argc, char **argv, opj_dparameters_t *parameters,i
 
 				/* ----------------------------------------------------- */
 			
-			default:
-				fprintf(stderr,"WARNING -> this option is not valid \"-%c %s\"\n",c, opj_optarg);
-				break;
+        default:
+            fprintf(stderr, "[WARNING] An invalid option has been ignored.\n");
+            break;
 		}
 	}while(c != -1);
 
 	/* check for possible errors */
 	if(img_fol->set_imgdir==1){
 		if(!(parameters->infile[0]==0)){
-			fprintf(stderr, "Error: options -ImgDir and -i cannot be used together !!\n");
+            fprintf(stderr, "[ERROR] options -ImgDir and -i cannot be used together.\n");
 			return 1;
 		}
 		if(img_fol->set_out_format == 0){
-			fprintf(stderr, "Error: When -ImgDir is used, -OutFor <FORMAT> must be used !!\n");
-			fprintf(stderr, "Only one format allowed! Valid format PGM, PPM, PNM, PGX, BMP, TIF, RAW and TGA!!\n");
+            fprintf(stderr, "[ERROR] When -ImgDir is used, -OutFor <FORMAT> must be used.\n");
+            fprintf(stderr, "Only one format allowed.\n"
+                            "Valid format are PGM, PPM, PNM, PGX, BMP, TIF, RAW and TGA.\n");
 			return 1;
 		}
 		if(!((parameters->outfile[0] == 0))){
-			fprintf(stderr, "Error: options -ImgDir and -o cannot be used together !!\n");
+            fprintf(stderr, "[ERROR] options -ImgDir and -o cannot be used together.\n");
 			return 1;
 		}
 	}else{
 		if((parameters->infile[0] == 0) || (parameters->outfile[0] == 0)) {
-			fprintf(stderr, "Example: %s -i image.j2k -o image.pgm\n",argv[0]);
-			fprintf(stderr, "    Try: %s -h\n",argv[0]);
+            fprintf(stderr, "[ERROR] Required parameters are missing\n"
+                            "Example: %s -i image.j2k -o image.pgm\n",argv[0]);
+            fprintf(stderr, "   Help: %s -h\n",argv[0]);
 			return 1;
 		}
 	}
@@ -639,8 +642,8 @@ int parse_DA_values( char* inArg, unsigned int *DA_x0, unsigned int *DA_y0, unsi
 		return EXIT_FAILURE;
 	}
 	else{
-		*DA_x0 = values[0]; *DA_y0 = values[1];
-		*DA_x1 = values[2]; *DA_y1 = values[3];
+		*DA_x0 = (OPJ_UINT32)values[0]; *DA_y0 = (OPJ_UINT32)values[1];
+		*DA_x1 = (OPJ_UINT32)values[2]; *DA_y1 = (OPJ_UINT32)values[3];
 		return EXIT_SUCCESS;
 	}
 }
@@ -676,8 +679,6 @@ static void info_callback(const char *msg, void *client_data) {
 /* -------------------------------------------------------------------------- */
 int main(int argc, char **argv)
 {
-	FILE *fsrc = NULL;
-
 	opj_dparameters_t parameters;			/* decompression parameters */
 	opj_image_t* image = NULL;
 	opj_stream_t *l_stream = NULL;				/* Stream */
@@ -689,6 +690,7 @@ int main(int argc, char **argv)
 	OPJ_INT32 num_images, imageno;
 	img_fol_t img_fol;
 	dircnt_t *dirptr = NULL;
+  int failed = 0;
 
 	/* set decoding parameters to default values */
 	opj_set_default_decoder_parameters(&parameters);
@@ -711,8 +713,8 @@ int main(int argc, char **argv)
 
 		dirptr=(dircnt_t*)malloc(sizeof(dircnt_t));
 		if(dirptr){
-			dirptr->filename_buf = (char*)malloc(num_images*OPJ_PATH_LEN*sizeof(char));	/* Stores at max 10 image file names*/
-			dirptr->filename = (char**) malloc(num_images*sizeof(char*));
+			dirptr->filename_buf = (char*)malloc((size_t)num_images*OPJ_PATH_LEN*sizeof(char));	/* Stores at max 10 image file names*/
+			dirptr->filename = (char**) malloc((size_t)num_images*sizeof(char*));
 
 			if(!dirptr->filename_buf){
 				return EXIT_FAILURE;
@@ -746,16 +748,10 @@ int main(int argc, char **argv)
 
 		/* read the input file and put it in memory */
 		/* ---------------------------------------- */
-		fsrc = fopen(parameters.infile, "rb");
-		if (!fsrc) {
-			fprintf(stderr, "ERROR -> failed to open %s for reading\n", parameters.infile);
-			return EXIT_FAILURE;
-		}
 
-		l_stream = opj_stream_create_default_file_stream(fsrc,1);
+		l_stream = opj_stream_create_default_file_stream(parameters.infile,1);
 		if (!l_stream){
-			fclose(fsrc);
-			fprintf(stderr, "ERROR -> failed to create the stream from the file\n");
+			fprintf(stderr, "ERROR -> failed to create the stream from the file %s\n", parameters.infile);
 			return EXIT_FAILURE;
 		}
 
@@ -794,9 +790,8 @@ int main(int argc, char **argv)
 
 		/* Setup the decoder decoding parameters using user parameters */
 		if ( !opj_setup_decoder(l_codec, &parameters) ){
-			fprintf(stderr, "ERROR -> j2k_dump: failed to setup the decoder\n");
+			fprintf(stderr, "ERROR -> opj_compress: failed to setup the decoder\n");
 			opj_stream_destroy(l_stream);
-			fclose(fsrc);
 			opj_destroy_codec(l_codec);
 			return EXIT_FAILURE;
 		}
@@ -806,7 +801,6 @@ int main(int argc, char **argv)
 		if(! opj_read_header(l_stream, l_codec, &image)){
 			fprintf(stderr, "ERROR -> opj_decompress: failed to read the header\n");
 			opj_stream_destroy(l_stream);
-			fclose(fsrc);
 			opj_destroy_codec(l_codec);
 			opj_image_destroy(image);
 			return EXIT_FAILURE;
@@ -814,13 +808,12 @@ int main(int argc, char **argv)
 
 		if (!parameters.nb_tile_to_decode) {
 			/* Optional if you want decode the entire image */
-			if (!opj_set_decode_area(l_codec, image, parameters.DA_x0,
-					parameters.DA_y0, parameters.DA_x1, parameters.DA_y1)){
+			if (!opj_set_decode_area(l_codec, image, (OPJ_INT32)parameters.DA_x0,
+					(OPJ_INT32)parameters.DA_y0, (OPJ_INT32)parameters.DA_x1, (OPJ_INT32)parameters.DA_y1)){
 				fprintf(stderr,	"ERROR -> opj_decompress: failed to set the decoded area\n");
 				opj_stream_destroy(l_stream);
 				opj_destroy_codec(l_codec);
 				opj_image_destroy(image);
-				fclose(fsrc);
 				return EXIT_FAILURE;
 			}
 
@@ -830,7 +823,6 @@ int main(int argc, char **argv)
 				opj_destroy_codec(l_codec);
 				opj_stream_destroy(l_stream);
 				opj_image_destroy(image);
-				fclose(fsrc);
 				return EXIT_FAILURE;
 			}
 		}
@@ -842,7 +834,6 @@ int main(int argc, char **argv)
 				opj_destroy_codec(l_codec);
 				opj_stream_destroy(l_stream);
 				opj_image_destroy(image);
-				fclose(fsrc);
 				return EXIT_FAILURE;
 			}*/
 
@@ -851,7 +842,6 @@ int main(int argc, char **argv)
 				opj_destroy_codec(l_codec);
 				opj_stream_destroy(l_stream);
 				opj_image_destroy(image);
-				fclose(fsrc);
 				return EXIT_FAILURE;
 			}
 			fprintf(stdout, "tile %d is decoded!\n\n", parameters.tile_index);
@@ -859,14 +849,20 @@ int main(int argc, char **argv)
 
 		/* Close the byte stream */
 		opj_stream_destroy(l_stream);
-		fclose(fsrc);
 
 		if(image->color_space == OPJ_CLRSPC_SYCC){
 			color_sycc_to_rgb(image); /* FIXME */
 		}
+		
+		if( image->color_space != OPJ_CLRSPC_SYCC 
+			&& image->numcomps == 3 && image->comps[0].dx == image->comps[0].dy
+			&& image->comps[1].dx != 1 )
+			image->color_space = OPJ_CLRSPC_SYCC;
+		else if (image->numcomps <= 2)
+			image->color_space = OPJ_CLRSPC_GRAY;
 
 		if(image->icc_profile_buf) {
-#if defined(HAVE_LIBLCMS1) || defined(HAVE_LIBLCMS2)
+#if defined(OPJ_HAVE_LIBLCMS1) || defined(OPJ_HAVE_LIBLCMS2)
 			color_apply_icc_profile(image); /* FIXME */
 #endif
 			free(image->icc_profile_buf);
@@ -878,81 +874,90 @@ int main(int argc, char **argv)
 		switch (parameters.cod_format) {
 		case PXM_DFMT:			/* PNM PGM PPM */
 			if (imagetopnm(image, parameters.outfile)) {
-				fprintf(stdout,"Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
 
 		case PGX_DFMT:			/* PGX */
 			if(imagetopgx(image, parameters.outfile)){
-				fprintf(stdout,"Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
 
 		case BMP_DFMT:			/* BMP */
 			if(imagetobmp(image, parameters.outfile)){
-				fprintf(stdout,"Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
-#ifdef HAVE_LIBTIFF
+#ifdef OPJ_HAVE_LIBTIFF
 		case TIF_DFMT:			/* TIFF */
 			if(imagetotif(image, parameters.outfile)){
-				fprintf(stdout,"Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
-#endif /* HAVE_LIBTIFF */
+#endif /* OPJ_HAVE_LIBTIFF */
 		case RAW_DFMT:			/* RAW */
 			if(imagetoraw(image, parameters.outfile)){
-				fprintf(stdout,"Error generating raw file. Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Error generating raw file. Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Successfully generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
 
 		case RAWL_DFMT:			/* RAWL */
 			if(imagetorawl(image, parameters.outfile)){
-				fprintf(stdout,"Error generating rawl file. Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Error generating rawl file. Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Successfully generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
 
 		case TGA_DFMT:			/* TGA */
 			if(imagetotga(image, parameters.outfile)){
-				fprintf(stdout,"Error generating tga file. Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Error generating tga file. Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Successfully generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
-#ifdef HAVE_LIBPNG
+#ifdef OPJ_HAVE_LIBPNG
 		case PNG_DFMT:			/* PNG */
 			if(imagetopng(image, parameters.outfile)){
-				fprintf(stdout,"Error generating png file. Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Error generating png file. Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 			}
 			else {
-				fprintf(stdout,"Successfully generated Outfile %s\n",parameters.outfile);
+                fprintf(stdout,"[INFO] Generated Outfile %s\n",parameters.outfile);
 			}
 			break;
-#endif /* HAVE_LIBPNG */
+#endif /* OPJ_HAVE_LIBPNG */
 /* Can happen if output file is TIFF or PNG
- * and HAVE_LIBTIF or HAVE_LIBPNG is undefined
+ * and OPJ_HAVE_LIBTIF or OPJ_HAVE_LIBPNG is undefined
 */
 			default:
-				fprintf(stderr,"Outfile %s not generated\n",parameters.outfile);
+                fprintf(stderr,"[ERROR] Outfile %s not generated\n",parameters.outfile);
+        failed = 1;
 		}
 
 		/* free remaining structures */
@@ -967,8 +972,9 @@ int main(int argc, char **argv)
 		/* destroy the codestream index */
 		opj_destroy_cstr_index(&cstr_index);
 
+		if(failed) remove(parameters.outfile);
 	}
-	return EXIT_SUCCESS;
+	return failed ? EXIT_FAILURE : EXIT_SUCCESS;
 }
 /*end main*/
 
